"""Clever, quick iterators that make your smile whiter."""

from ._fun import *
from ._ytr import *
from ._key import *

__all__ = [
    "yany",
    "yall",
    "ylen",
    "last",
    "head",
    "tail",
    "finish",
    "minmax",
    "minmedmax",
    "isiter",
    "contain",
    "repeat",
    "sequence",
    "call",
    "percent",
    "flat",
    "chunk",
    "key",
    "unique",
    "duplicates",
    "formatter",
    "numeric",
    "getter",
]
